/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "grid.h"

void gridInit(grid_t *grid) {
  grid->breakpointCount = 0x00;
  grid->breakpointCurrent = 0x00;
  grid->childCount = 0x00;
  grid->width = 0;
  grid->height = 0;
  grid->onResize = NULL;
}

uint8_t gridAddBreakpoint(
  grid_t *grid, float width,
  uint8_t columns, uint8_t rows,
  float gutterX, float gutterY
) {
  uint8_t i = grid->breakpointCount++;

  grid->breakpoints[i].rows = rows;
  grid->breakpoints[i].columns = columns;
  grid->breakpoints[i].gutterX = gutterX;
  grid->breakpoints[i].gutterY = gutterY;
  grid->breakpointSizes[i].width = width;

  return i;
}

gridchild_t * gridAddChild(grid_t *grid) {
  gridchild_t *child = grid->children + grid->childCount++;
  child->breakpointCount = 0;
  return child;
}


uint8_t gridChildAddBreakpoint(gridchild_t *child,
  uint8_t x, uint8_t y, uint8_t columns, uint8_t rows
) {
  uint8_t i;
  gridchildbreakpoint_t *bp;
  i = child->breakpointCount++;
  bp = child->breakpoints + i;
  bp->x = x;
  bp->y = y;
  bp->rows = rows;
  bp->columns = columns;
  return i;
}

gridchildbreakpoint_t * gridChildGetBreakpoint(gridchild_t *child, uint8_t bp) {
  return child->breakpoints + (
    bp >= child->breakpointCount ? child->breakpointCount - 1 : bp
  );
}

uint8_t gridGetBreakpoint(grid_t *grid, float screenWidth) {
  uint8_t i;
  for(i = 0; i < grid->breakpointCount; i++) {
    if(grid->breakpointSizes[i].width < screenWidth) continue;
    return i;
  }
  return grid->breakpointCount - 1;
}

void gridSetSize(grid_t *grid,
  float screenWidth, float screenHeight,
  float width, float height,
  float x, float y
) {
  uint8_t i, breakpoint;
  gridbreakpoint_t *gridbp;
  float sizeCol, sizeRow, gx, gy, gw, gh;

  // Need to resize?
  if(
    grid->width == width && grid->height == height &&
    grid->x == x && grid->y == y
  ) {
    return;
  }

  // Update properties
  grid->width = width;
  grid->height = height;
  grid->x = x;
  grid->y = y;

  // Determine breakpoint
  breakpoint = gridGetBreakpoint(grid, screenWidth);
  gridbp = grid->breakpoints + breakpoint;
  grid->breakpointCurrent = breakpoint;

  if(grid->onResize == NULL) return;

  sizeCol = -1;
  sizeRow = -1;

  // Resize children
  for(i = 0; i < grid->childCount; i++) {
    // Get the item and the definition.

    gridGetChildSize(
      grid, breakpoint, grid->children + i,
      &sizeCol, &sizeRow, &gx, &gy, &gw, &gh
    );

    // Fire the resize event.
    grid->onResize(
      grid->user, i, gridChildGetBreakpoint(grid->children + i, breakpoint),
      screenWidth, screenHeight,
      gx, gy, gw, gh
    );
  }
}

void gridGetChildSize(
  grid_t *grid, uint8_t breakpoint, gridchild_t *child,
  float *sizeCol, float *sizeRow,
  float *x, float *y, float *width, float *height
) {
  gridchildbreakpoint_t *childbp;
  gridbreakpoint_t *gridbp;

  gridbp = grid->breakpoints + breakpoint;
  
  if(*sizeCol < 0) {
    *sizeCol = (
      grid->width - (gridbp->gutterX * (gridbp->columns-1))
    ) / gridbp->columns;
  }

  if(*sizeRow < 0) {
    *sizeRow = (
      grid->height - (gridbp->gutterY * (gridbp->rows - 1))
    ) / gridbp->rows;
  }

  childbp = gridChildGetBreakpoint(child, breakpoint);

  *x = (*sizeCol * childbp->x) + (gridbp->gutterX * childbp->x) + grid->x;
  *y = (*sizeRow * childbp->y) + (gridbp->gutterY * childbp->y) + grid->y;

  *width = (
    (childbp->columns * *sizeCol) + 
    (mathMax(childbp->columns - 1, 0) * gridbp->gutterX)
  );
  *height = (
    (childbp->rows * *sizeRow) + 
    (mathMax(childbp->rows - 1, 0) * gridbp->gutterY)
  );
}


align_t gridAlignChild(
  grid_t *grid, uint8_t breakpoint, gridchild_t *child,
  float *sizeCol, float *sizeRow,
  uint8_t alignX, uint8_t alignY,
  float width, float height
) {
  float gridWidth, gridHeight, gridX, gridY;
  align_t alignment;

  // Get the child size.
  gridGetChildSize(
    grid, breakpoint, child, sizeCol, sizeRow,
    &gridX, &gridY, &gridWidth, &gridHeight
  );

  if(width == -1) width = gridWidth;
  if(height == -1) height = gridHeight;

  // Align the child
  alignment = alignmentGet(
    alignX, alignY, gridWidth, gridHeight, width, height, -1, -1
  );

  alignment.x += gridX;
  alignment.y += gridY;

  return alignment;
}