/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>
#include "align.h"

/**
 * Initialize a grid system.
 * 
 * @param grid Grid system to initialize.
 */
void gridInit(grid_t *grid);

/**
 * Adds a breakpoint to a grid system.
 * 
 * @param grid Grid to add to.
 * @param width Screen dimension width of the breakpoint.
 * @param columns How many columns at this breakpoint.
 * @param rows How many rows at this breakpoint.
 * @param gutterX Gutter X at this breakpoint.
 * @param gutterY Gutter Y at this breakpoint.
 * @return Index of the breakpoint within the grid.
 */
uint8_t gridAddBreakpoint(
  grid_t *grid, float width,
  uint8_t columns, uint8_t rows,
  float gutterX, float gutterY
);

/**
 * Add a child to the grid.
 * 
 * @param grid Grid to add the child to.
 * @return The grid child item that was added.
 */
gridchild_t * gridAddChild(grid_t *grid);


/**
 * Add a breakpoint to a grid child.
 * 
 * @param child Child to add the breakpoint to.
 * @param x Column X position.
 * @param y Row Y position.
 * @param columns Count of columns this child will span.
 * @param rows Count of rows this child will span.
 * @return The index to the breakpoint in question.
 */
uint8_t gridChildAddBreakpoint(gridchild_t *child,
  uint8_t x, uint8_t y, uint8_t columns, uint8_t rows
);

/**
 * Get the breakpoint (index) from a grid and screen width.
 * 
 * @param grid Grid to get from.
 * @param screenWidth Screen width to use.
 * @return Grid index.
 */
uint8_t gridGetBreakpoint(grid_t *grid, float screenWidth);

/**
 * Retreive the breakpoint to use for a child. Takes missing breakpoints into
 * consideration.
 * 
 * @param child Child to get the breakpoint from.
 * @param bp Breakpoint index to use.
 * @return The childs matching breakpoint
 */
gridchildbreakpoint_t * gridChildGetBreakpoint(gridchild_t *child, uint8_t bp);

/**
 * Set the size of a grid system. This will only update if it's found to be 
 * necessary.
 * 
 * @param grid Grid system to resize.
 * @param screenWidth Current screen width.
 * @param screenHeight Current screen height.
 * @param width Width of the grid itself, useful for nested grids.
 * @param height Height of the grid itself, useful for nested grids.
 * @param x X position of this grid (to offset children by). Only for events.
 * @param y Y position of this grid (to offset children by). Only for events.
 */
void gridSetSize(grid_t *grid,
  float screenWidth, float screenHeight,
  float width, float height,
  float x, float y
);

/**
 * Get the size of a grid child.
 * 
 * @param grid Grid to get from.
 * @param breakpoint Breakpoint index to get.
 * @param child Grid child to get for.
 * @param sizeCol Pointer to a float that will/already holds the column size.
 * @param sizeRow Poitner to a float that will/aready holds the row size.
 * @param x Pointer to the output X position.
 * @param y Pointer to the output Y position.
 * @param width Pointer to the output width.
 * @param height Pointer to the output height.
 */
void gridGetChildSize(
  grid_t *grid, uint8_t breakpoint, gridchild_t *child,
  float *sizeCol, float *sizeRow,
  float *x, float *y, float *width, float *height
);

/**
 * Align a grid item child.
 * 
 * @param grid Grid to align within.
 * @param breakpoint Breakpoint index.
 * @param child Grid child.
 * @param sizeCol Pointer to a float that will/already holds the column size.
 * @param sizeRow Poitner to a float that will/aready holds the row size.
 * @param alignX Alignment options for the X axis.
 * @param alignY Alignment options for the Y axis.
 * @param width Width of the child item to be aligned.
 * @param height Height of the child item to be aligned.
 * @return The alignment within the grid cell.
 */
align_t gridAlignChild(
  grid_t *grid, uint8_t breakpoint, gridchild_t *child,
  float *sizeCol, float *sizeRow,
  uint8_t alignX, uint8_t alignY,
  float width, float height
);